import numpy as np

from escape.utils.widgets.base import PlotObjectLayout
from escape.utils.widgets.plot import ErrorPlot2dWidget, MapWidget, PLOT_TYPE_KW, \
    PLOT2D_TYPE, MAP_TYPE, PLOT_TYPES


class DataObjectLayout(PlotObjectLayout):
    
    def update(self, *args):
        escobj = self.escobj
        domain_size = escobj.domain_size
        xarr = np.array(escobj.ini_coordinates)
        yarr = np.array(escobj.ini_intensities)
        earr = np.array(escobj.ini_errors)
        marr = np.array(escobj.mask)
        if domain_size == 1: 
            if not len(earr):
                earr = np.sqrt(yarr)
            self.center.update(np.ma.masked_array(xarr, ~marr),
                               np.ma.masked_array(yarr, ~marr),
                               np.ma.masked_array(earr, ~marr))
        else:
            plot_type = self._plot_type
            coordidx = self._coordidx
            if plot_type == PLOT2D_TYPE:
                if not isinstance(coordidx, int):
                    raise TypeError("Unexpected type of coordinate index, expected int, got {0}".format(
                        type(coordidx)
                    ))
                xarr = xarr[coordidx::domain_size]
                self.center.update(np.ma.masked_array(xarr, ~marr),
                               np.ma.masked_array(yarr, ~marr),
                               np.ma.masked_array(earr, ~marr))
            elif plot_type == MAP_TYPE:
                if domain_size > 2:
                    if not isinstance(coordidx, (list, tuple)):
                        raise TypeError("Unexpected type of coordinate index, expected list or tuple, got {0}".format(
                            type(coordidx)
                        ))
                    
                    msk = np.zeros(shape=xarr.shape, dtype=bool)
                    
                    for idx in coordidx:
                        msk[idx::domain_size] = True
                    xarr = xarr[msk]
                rows = self._rows
                cols = int(len(yarr) / rows)
                if rows * cols != len(yarr):
                    raise ValueError("Cannot build map with given number of rows")
                yarr = yarr.reshape((rows, cols))
                marr = marr.reshape((rows, cols))
                xarr = xarr.reshape((int(len(xarr) / 2), 2))
                xmin, xmax = np.min(xarr[:, 0]), np.max(xarr[:, 0])
                ymin, ymax = np.min(xarr[:, 1]), np.max(xarr[:, 1])
                self.center.update(np.ma.masked_array(yarr, ~marr), extent=(xmin, xmax, ymin, ymax))
            else:
                raise ValueError("Wrong type of plot, expected one of [{0}], got {1}".format(
                    ", ".join(PLOT_TYPES), plot_type
                ))
